package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.entity.Loss;
import com.tofly.yswzMaterials.entity.LossDetail;
import com.tofly.yswzMaterials.entity.StorageDetail;
import com.tofly.yswzMaterials.entity.vo.*;
import com.tofly.yswzMaterials.mapper.LossMapper;
import com.tofly.yswzMaterials.service.LossDetailService;
import com.tofly.yswzMaterials.service.LossService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class LosServiceImpl extends ServiceImpl<LossMapper, Loss> implements LossService {

    private final CallApiService callApiService;
    private final LossDetailService lossDetailService;

    /**
     * 添加盘亏单
     *
     * @param loss 盘亏
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertLoss(Loss loss) {
        List<LossDetail> detailList = loss.getDetailList();
        //添加入库单
        save(loss);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setLossId(loss.getId());
            lossDetailService.save(detail);
        });
        List<LossDetailParamVo> detailVoList = new ArrayList<>();
        //格式转换
        for (LossDetail lossDetail : detailList) {
            LossDetailParamVo vo = new LossDetailParamVo();
            BeanUtils.copyProperties(lossDetail,vo);
            detailVoList.add(vo);
        }
        LossParamVo lossParamVo = new LossParamVo();
        loss.setDetailList(null);
        BeanUtils.copyProperties(loss,lossParamVo);
        lossParamVo.setDetailList(detailVoList);
        lossParamVo.setFDate(DateUtil.format(loss.getFDate(),"yyyy-MM-dd HH:mm:ss"));
        //body信息
        String body = JSONObject.toJSONString(lossParamVo);
        //获取token信息
        String tokenInfo = callApiService.getToken();
        //获取url地址
        String url = JinDieUrlEnum.SAVE_UPDATE_LOSS_URL.getName() + "?token=" + tokenInfo;
        Map apiResult = callApiService.getApiResult(body, url);
        System.out.println(JSONObject.toJSONString(apiResult));
        return true;
    }

    /**
     * 查询盘亏单
     *
     * @param code code
     * @return LossParamVo
     */
    @Override
    public LossParamVo getLossInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_LOSS_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), LossParamVo.class);
        }
        return null;
    }


}
