package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.entity.Picking;
import com.tofly.yswzMaterials.entity.PickingDetail;
import com.tofly.yswzMaterials.entity.vo.PickingDetailParamVo;
import com.tofly.yswzMaterials.entity.vo.PickingParamVo;
import com.tofly.yswzMaterials.entity.vo.StorageParamVo;
import com.tofly.yswzMaterials.mapper.PickingMapper;
import com.tofly.yswzMaterials.service.PickingDetailService;
import com.tofly.yswzMaterials.service.PickingService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class PickingServiceImpl extends ServiceImpl<PickingMapper, Picking> implements PickingService {

    private final PickingDetailService pickingDetailService;
    private final CallApiService callApiService;

    /**
     * 新增外购入库单
     *
     * @param picking picking
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertPicking(Picking picking) {
        List<PickingDetail> detailList = picking.getDetailList();
        //添加入库单
        save(picking);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setPickingId(picking.getId());
            pickingDetailService.save(detail);
        });
        List<PickingDetailParamVo> detailVoList = new ArrayList<>();
        //格式转换
        for (PickingDetail pickingDetail : detailList) {
            PickingDetailParamVo vo = new PickingDetailParamVo();
            BeanUtils.copyProperties(pickingDetail,vo);
            detailVoList.add(vo);
        }
        PickingParamVo pickingParam = new PickingParamVo();
        picking.setDetailList(null);
        BeanUtils.copyProperties(picking,pickingParam);
        pickingParam.setDetailList(detailVoList);
        pickingParam.setFDate(DateUtil.format(picking.getFDate(),"yyyy-MM-dd HH:mm:ss"));
        //body信息
        String body = JSONObject.toJSONString(pickingParam);
        //获取token信息
        String tokenInfo = callApiService.getToken();
        //获取url地址
        String url = JinDieUrlEnum.SAVE_UPDATE_PICKING_URL.getName() + "?token=" + tokenInfo;
        Map apiResult = callApiService.getApiResult(body, url);
        System.out.println(JSONObject.toJSONString(apiResult));
        return true;
    }

    /**
     * 获取出库领料信息
     *
     * @param code 编码
     * @return PickingParamVo
     */
    @Override
    public PickingParamVo getPickingInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_PICKING_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), PickingParamVo.class);
        }
        return null;
    }
}
