package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.entity.PickingDetail;
import com.tofly.yswzMaterials.entity.Profit;
import com.tofly.yswzMaterials.entity.ProfitDetail;
import com.tofly.yswzMaterials.entity.vo.PickingDetailParamVo;
import com.tofly.yswzMaterials.entity.vo.PickingParamVo;
import com.tofly.yswzMaterials.entity.vo.ProfitDetailParamVo;
import com.tofly.yswzMaterials.entity.vo.ProfitParamVo;
import com.tofly.yswzMaterials.mapper.ProfitMapper;
import com.tofly.yswzMaterials.service.ProfitDetailService;
import com.tofly.yswzMaterials.service.ProfitService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class ProfitServiceImpl extends ServiceImpl<ProfitMapper, Profit> implements ProfitService {

    private final CallApiService callApiService;
    private final ProfitDetailService profitDetailService;

    /**
     * 插入盘盈单
     *
     * @param profit 盘盈
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertProfit(Profit profit) {
        List<ProfitDetail> detailList = profit.getDetailList();
        //添加入库单
        save(profit);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setProfitId(profit.getId());
            profitDetailService.save(detail);
        });
        List<ProfitDetailParamVo> detailVoList = new ArrayList<>();
        //格式转换
        for (ProfitDetail profitDetail : detailList) {
            ProfitDetailParamVo vo = new ProfitDetailParamVo();
            BeanUtils.copyProperties(profitDetail,vo);
            detailVoList.add(vo);
        }
        ProfitParamVo profitParamVo = new ProfitParamVo();
        profit.setDetailList(null);
        BeanUtils.copyProperties(profit,profitParamVo);
        profitParamVo.setDetailList(detailVoList);
        profitParamVo.setFDate(DateUtil.format(profit.getFDate(),"yyyy-MM-dd HH:mm:ss"));
        //body信息
        String body = JSONObject.toJSONString(profitParamVo);
        //获取token信息
        String tokenInfo = callApiService.getToken();
        //获取url地址
        String url = JinDieUrlEnum.SAVE_UPDATE_PROFIT_URL.getName() + "?token=" + tokenInfo;
        Map apiResult = callApiService.getApiResult(body, url);
        System.out.println(JSONObject.toJSONString(apiResult));
        return true;
    }

    /**
     * 查询盘盈单信息
     *
     * @param code code
     * @return ProfitParamVo
     */
    @Override
    public ProfitParamVo getProfitInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_PROFIT_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), ProfitParamVo.class);
        }
        return null;
    }
}
