package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.yswzMaterials.em.Constant;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.*;
import com.tofly.yswzMaterials.entity.vo.InStockCountVo;
import com.tofly.yswzMaterials.mapper.StocksAuditMapper;
import com.tofly.yswzMaterials.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author bubble
 */
@Service
public class StocksAuditServiceImpl extends ServiceImpl<StocksAuditMapper, StocksAudit>
    implements StocksAuditService {

  @Autowired private StocksAuditConfigService stocksAuditConfigService;
  @Autowired private StocksInstockInfoService stocksInstockInfoService;
  @Autowired private StocksInstockDetailService stocksInstockDetailService;
  @Autowired private StocksOutstockInfoService stocksOutstockInfoService;
  @Autowired private StocksOutstockDetailService stocksOutstockDetailService;
  @Autowired private StocksTakeawayInfoService stocksTakeawayInfoService;
  @Autowired private StocksTakeawayDetailService stocksTakeawayDetailService;
  @Autowired private StocksStockInfoService stockInfoService;

  @Value("${yswz.ftp.auditFile}")
  private String auditFile;

  /*  private static final String MESSAGE_PUSH = "TF_YWPN_MESSAGEPUSH_W";
  private static final String MESSAGE_RECIPIENT = "TF_YWPN_MESSAGERECIPIENT_W";*/

  /**
   * 发起审核
   *
   * @param type 审核类型：出库、入库等，取字典表
   * @param id 审核表id
   * @return Boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean initiateReview(Long type, Long id) {
    if (type == null) {
      throw new ToflyDeniedException("未选择配置类型");
    }
    if (id == null) {
      throw new ToflyDeniedException("未选择审核上报对象");
    }

    // 获取对应类型的审核配置的一级审核人
    StocksAuditConfig auditConfig =
        stocksAuditConfigService.getOne(
            Wrappers.<StocksAuditConfig>lambdaQuery()
                .eq(StocksAuditConfig::getType, type)
                .eq(StocksAuditConfig::getAuditLevel, Constant.ONE)
                .orderByAsc(StocksAuditConfig::getAuditLevel));

    if (auditConfig == null) {
      throw new ToflyDeniedException("请确定该模块是否已配置审批流程");
    }

    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode())) {
      StocksInstockInfo instockInfo = stocksInstockInfoService.getById(id);
      if (instockInfo == null) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long instockStatus = instockInfo.getInstockStatus();
      if (!instockStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
          && !instockStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      // 修改出库表的当前审核人
      stocksInstockInfoService.update(
          Wrappers.<StocksInstockInfo>lambdaUpdate()
              .eq(StocksInstockInfo::getId, instockInfo.getId())
              .set(StocksInstockInfo::getInstockStatus, StocksAuditEnum.UNDER_REVIEW.getCode())
              .set(StocksInstockInfo::getAuditUser, auditConfig.getAuditUser())
              .set(StocksInstockInfo::getAuditDept, auditConfig.getAuditDept()));
    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())) {
      StocksOutstockInfo outStockInfo = stocksOutstockInfoService.getById(id);
      if (outStockInfo == null) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long outstockStatus = outStockInfo.getOutstockStatus();
      if (!outstockStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
          && !outstockStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      // 修改出库表的当前审核人
      stocksOutstockInfoService.update(
          Wrappers.<StocksOutstockInfo>lambdaUpdate()
              .eq(StocksOutstockInfo::getId, outStockInfo.getId())
              .set(StocksOutstockInfo::getOutstockStatus, StocksAuditEnum.UNDER_REVIEW.getCode())
              .set(StocksOutstockInfo::getAuditUser, auditConfig.getAuditUser())
              .set(StocksOutstockInfo::getAuditDept, auditConfig.getAuditDept()));
    } else {
      StocksTakeawayInfo takeawayInfo = stocksTakeawayInfoService.getById(id);
      if (takeawayInfo == null) {
        throw new ToflyDeniedException("未找到该记录");
      }
      Long takeawayStatus = takeawayInfo.getTakeawayStatus();
      if (!takeawayStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
          && !takeawayStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
        throw new ToflyDeniedException("该记录已审核完成或正常审核，无法发起审批");
      }
      // 修改出库表的当前审核人
      stocksTakeawayInfoService.update(
          Wrappers.<StocksTakeawayInfo>lambdaUpdate()
              .eq(StocksTakeawayInfo::getId, takeawayInfo.getId())
              .set(StocksTakeawayInfo::getTakeawayStatus, StocksAuditEnum.UNDER_REVIEW.getCode())
              .set(StocksTakeawayInfo::getAuditUser, auditConfig.getAuditUser())
              .set(StocksTakeawayInfo::getAuditDept, auditConfig.getAuditDept()));
    }
    List<StocksAudit> auditList = list(Wrappers.<StocksAudit>lambdaQuery()
                .eq(StocksAudit::getAuditType, type)
                .eq(StocksAudit::getTableId, id)
                .select(StocksAudit::getSort)
                .orderByDesc(StocksAudit::getId));
    // 赋值
    StocksAudit stocksAudit = new StocksAudit();
    stocksAudit.setAuditLevel(auditConfig.getAuditLevel());
    stocksAudit.setAuditDept(auditConfig.getAuditDept());
    stocksAudit.setAuditUser(auditConfig.getAuditUser());
    stocksAudit.setAuditType(auditConfig.getType());
    stocksAudit.setTableId(id);
    stocksAudit.setSort(CollectionUtils.isEmpty(auditList) ? Constant.ONE : auditList.get(0).getSort());
    stocksAudit.setAuditStatus(StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode());
    stocksAudit.setTableName(getTableByType(type));
    // 添加一级审核人信息
    save(stocksAudit);
    return true;
  }

  /**
   * 获取当前记录的最新审批信息
   *
   * @param type 审批类型
   * @param id id
   * @return StocksAudit
   */
  @Override
  public StocksAudit nowAuditInfo(Long type, Long id) {
    return getOne(
        Wrappers.<StocksAudit>lambdaQuery()
            .eq(StocksAudit::getTableId, id)
            .eq(StocksAudit::getAuditStatus, StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode())
            .eq(StocksAudit::getTableName, getTableByType(type))
            .eq(StocksAudit::getAuditUser, SecurityUtils.getUserId())
            .eq(StocksAudit::getAuditType, type));
  }

  /**
   * 用户审批
   *
   * @param audit 审批信息
   * @return boolean
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean userApprovals(StocksAudit audit, List<MultipartFile> fileIds) {
    if (audit == null) {
      throw new ToflyDeniedException("审核信息为空");
    }

    StocksAudit stockAudit =
        getOne(
            Wrappers.<StocksAudit>lambdaQuery()
                .eq(StocksAudit::getTableId, audit.getTableId())
                .eq(
                    StocksAudit::getAuditStatus,
                    StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode())
                .eq(StocksAudit::getAuditType, audit.getAuditType()));

    if (!stockAudit.getAuditUser().equals(SecurityUtils.getUserId())) {
      throw new ToflyDeniedException("您无权审核该记录");
    }
    stockAudit.setAuditStatus(audit.getAuditStatus());
    stockAudit.setAuditMsg(audit.getAuditMsg());

    Long auditType = stockAudit.getAuditType();
    Long auditLevel = stockAudit.getAuditLevel();
    Long auditStatus = stockAudit.getAuditStatus();
    Long tableId = stockAudit.getTableId();
    stockAudit.setAuditTime(new Date());
    // 当前审批记录存
    updateById(stockAudit);
    // 如果审批通过
    if (auditStatus.equals(StocksAuditEnum.AUDIT_STATUS_PASS.getCode())) {
      // 查询配置表是否存在下一级审核人
      StocksAuditConfig config =
          stocksAuditConfigService.getOne(
              Wrappers.<StocksAuditConfig>lambdaQuery()
                  .eq(StocksAuditConfig::getType, auditType)
                  .eq(StocksAuditConfig::getAuditLevel, auditLevel + 1));
      // 为空则不需审核，记录审批完成
      if (config == null) {
        updateInventory(auditType, tableId);
      } else {
        // 有则继续审核生成下一条审核信息
        handleAudit(stockAudit, config);
      }
    } else if (auditStatus.equals(StocksAuditEnum.AUDIT_STATUS_HSIANG_HSIEN.getCode())) {
      // 驳回查询上一级的配置数据
      StocksAuditConfig config =
          stocksAuditConfigService.getOne(
              Wrappers.<StocksAuditConfig>lambdaQuery()
                  .eq(StocksAuditConfig::getType, auditType)
                  .eq(StocksAuditConfig::getAuditLevel, auditLevel - 1));
      reviewRejection(config, auditType, tableId);
      // 外卖审核经驳回直接驳回不返回上一级，所以不生成审核记录表
      if (!auditType.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode()) && config != null) {
        handleAudit(stockAudit, config);
      }
    }
    return true;
  }

  // 增加审核信息
  private void handleAudit(StocksAudit audit, StocksAuditConfig config) {
    Long auditType = audit.getAuditType();
    Long tableId = audit.getTableId();
    Long auditDept = config.getAuditDept();
    Long auditUser = config.getAuditUser();
    StocksAudit stocksAudit = new StocksAudit();
    stocksAudit.setAuditLevel(config.getAuditLevel());
    stocksAudit.setAuditDept(auditDept);
    stocksAudit.setAuditUser(auditUser);
    stocksAudit.setAuditType(auditType);
    stocksAudit.setTableId(tableId);
    stocksAudit.setSort(audit.getSort() + 1);
    stocksAudit.setTableName(getTableByType(auditType));
    stocksAudit.setAuditStatus(StocksAuditEnum.AUDIT_STATUS_TO_BE_REVIEWED.getCode());
    save(stocksAudit);

    if (auditType.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode())) {
      stocksInstockInfoService.update(
          Wrappers.<StocksInstockInfo>lambdaUpdate()
              .eq(StocksInstockInfo::getId, tableId)
              .set(StocksInstockInfo::getAuditUser, auditUser)
              .set(StocksInstockInfo::getAuditDept, auditDept));
    } else if (auditType.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())) {
      stocksOutstockInfoService.update(
          Wrappers.<StocksOutstockInfo>lambdaUpdate()
              .eq(StocksOutstockInfo::getId, tableId)
              .set(StocksOutstockInfo::getAuditUser, auditUser)
              .set(StocksOutstockInfo::getAuditDept, auditDept));
    } else {
      stocksTakeawayInfoService.update(
          Wrappers.<StocksTakeawayInfo>lambdaUpdate()
              .eq(StocksTakeawayInfo::getId, tableId)
              .set(StocksTakeawayInfo::getAuditUser, auditUser)
              .set(StocksTakeawayInfo::getAuditDept, auditDept));
    }
  }

  /**
   * 更新库存
   *
   * @param type 类型
   * @param id 记录id
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public void updateInventory(Long type, Long id) {
    Long approved = StocksAuditEnum.APPROVED.getCode();
    // 入库类型操作
    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode())) {
      // 当前记录库存统计
      List<InStockCountVo> inStockCount = baseMapper.inStockInfoCount(id);
      if (CollectionUtils.isNotEmpty(inStockCount)) {
        for (InStockCountVo vo : inStockCount) {
          Double num = vo.getNum();
          // 总库存记录
          List<StocksStockInfo> stockInfo =
              stockInfoService.list(
                  Wrappers.<StocksStockInfo>lambdaQuery()
                      .eq(StocksStockInfo::getStoreId, vo.getStoreId())
                      .eq(StocksStockInfo::getDeviceId, vo.getDeviceId())
                      .eq(StocksStockInfo::getShelvesId, vo.getShelvesId()));
          StocksStockInfo ssi = new StocksStockInfo();
          // 信息不为空则更新库存信息
          if (!CollectionUtils.isEmpty(stockInfo)) {
            ssi.setId(stockInfo.get(0).getId());
            ssi.setStockNum(stockInfo.get(0).getStockNum() + num);
            ssi.setInstockTotalNum(stockInfo.get(0).getInstockTotalNum() + num);
          } else {
            ssi.setShelvesId(vo.getShelvesId());
            ssi.setDeviceId(vo.getDeviceId());
            ssi.setStoreId(vo.getStoreId());
            ssi.setStockNum(num);
            ssi.setInstockTotalNum(num);
            stockInfoService.save(ssi);
          }
          stockInfoService.saveOrUpdate(ssi);
          //入库明细保存库存id
          stocksInstockDetailService.update(Wrappers.<StocksInstockDetail>lambdaUpdate()
            .set(StocksInstockDetail::getStockInfoId,ssi.getId())
            .eq(StocksInstockDetail::getInstockInfoId,id)
            .eq(StocksInstockDetail::getDeviceId,vo.getDeviceId())
            .eq(StocksInstockDetail::getShelvesId,vo.getShelvesId()));
        }
      }
      // 清空入库记录表保存的审核信息
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.IN_STOCK_CONFIG_TABLE.getName(),
          id,
          StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode(),
          approved);

    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())) {
      // 出库类型操作
      List<StocksOutstockDetail> outStockDetails =
          stocksOutstockDetailService.list(
              Wrappers.<StocksOutstockDetail>lambdaQuery()
                  .eq(StocksOutstockDetail::getOutstockInfoId, id));

      if (CollectionUtils.isNotEmpty(outStockDetails)) {
        List<StocksStockInfo> stockInfoList = new ArrayList<>();
        for (StocksOutstockDetail outStockDetail : outStockDetails) {
          Double num = outStockDetail.getOutstockNum();
          Long stockInfoId = outStockDetail.getStockInfoId();
          StocksStockInfo stockInfo = stockInfoService.getById(stockInfoId);
          stockInfo.setOutstockTotalNum(stockInfo.getOutstockTotalNum() + num);
          stockInfo.setStockNum(stockInfo.getStockNum() - num);
          stockInfoList.add(stockInfo);
        }
        stockInfoService.updateBatchById(stockInfoList);
      }

      // 修改出库记录已完成审核
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.OUT_STOCK_CONFIG_TABLE.getName(),
          id,
          StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode(),
          approved);
    } else if (type.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())) {
      List<StocksTakeawayDetail> detailList =
          stocksTakeawayDetailService.list(
              Wrappers.<StocksTakeawayDetail>lambdaQuery().eq(StocksTakeawayDetail::getTakeawayId, id));
      if (CollectionUtils.isNotEmpty(detailList)) {
        StocksStockInfo stockInfo;
        for (StocksTakeawayDetail takeawayDetail : detailList) {
          Double num = takeawayDetail.getTakeawayNum();
          Long stockInfoId = takeawayDetail.getStockInfoId();
          stockInfo = stockInfoService.getById(stockInfoId);
          stockInfo.setTakeawayTotalNum(stockInfo.getTakeawayTotalNum() + num);
          stockInfo.setStockNum(stockInfo.getStockNum() - num);
          stockInfoService.updateById(stockInfo);
        }

      }
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TABLE.getName(),
          id,
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode(),
          approved);
    }
  }

  /**
   * 审核驳回
   *
   * @param type 类型
   * @param config 配置信息
   * @param id id
   */
  @Override
  public void reviewRejection(StocksAuditConfig config, Long type, Long id) {
    Long singHsien = StocksAuditEnum.HSIANG_HSIEN.getCode();

    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode())) {
      if (config == null) {
        // 重置入库未未审核状态
        baseMapper.emptyAuditInfo(
            StocksAuditEnum.IN_STOCK_CONFIG_TABLE.getName(),
            id,
            StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode(),
            singHsien);
      } else {
        // 修改入库审核信息
        stocksInstockInfoService.update(
            Wrappers.<StocksInstockInfo>lambdaUpdate()
                .eq(StocksInstockInfo::getId, id)
                .set(StocksInstockInfo::getAuditUser, config.getAuditUser())
                .set(StocksInstockInfo::getAuditDept, config.getAuditDept()));
      }
    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())) {
      if (config == null) {
        // 重置出库未审核状态
        baseMapper.emptyAuditInfo(
            StocksAuditEnum.OUT_STOCK_CONFIG_TABLE.getName(),
            id,
            StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode(),
            singHsien);
      } else {
        // 修改入库审核信息
        stocksOutstockInfoService.update(
            Wrappers.<StocksOutstockInfo>lambdaUpdate()
                .eq(StocksOutstockInfo::getId, id)
                .set(StocksOutstockInfo::getAuditUser, config.getAuditUser())
                .set(StocksOutstockInfo::getAuditDept, config.getAuditDept()));
      }

    } else if (type.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())) {
      baseMapper.emptyAuditInfo(
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TABLE.getName(),
          id,
          StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode(),
          singHsien);
    }
  }


  public static String getTableByType(Long type) {
    String tableName = null;
    if (type.equals(StocksAuditEnum.IN_STOCK_CONFIG_TYPE.getCode())) {
      tableName = StocksAuditEnum.IN_STOCK_CONFIG_TABLE.getName();
    } else if (type.equals(StocksAuditEnum.OUT_STOCK_CONFIG_TYPE.getCode())) {
      tableName = StocksAuditEnum.OUT_STOCK_CONFIG_TABLE.getName();
    } else if (type.equals(StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())) {
      tableName = StocksAuditEnum.TAKEAWAY_OUT_STOCK_TABLE.getName();
    }
    return tableName;
  }
}
