package com.tofly.yswzMaterials.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.yswzMaterials.em.StocksAuditEnum;
import com.tofly.yswzMaterials.entity.StocksAudit;
import com.tofly.yswzMaterials.entity.StocksTakeawayDetail;
import com.tofly.yswzMaterials.entity.StocksTakeawayInfo;
import com.tofly.yswzMaterials.entity.dto.StocksTakeawayInfoDto;
import com.tofly.yswzMaterials.mapper.StocksTakeawayInfoMapper;
import com.tofly.yswzMaterials.service.IFileInfoService;
import com.tofly.yswzMaterials.service.StocksAuditService;
import com.tofly.yswzMaterials.service.StocksTakeawayDetailService;
import com.tofly.yswzMaterials.service.StocksTakeawayInfoService;
import org.apache.commons.lang.time.DateFormatUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author bubble
 */
@Service
public class StocksTakeawayInfoServiceImpl
    extends ServiceImpl<StocksTakeawayInfoMapper, StocksTakeawayInfo>
    implements StocksTakeawayInfoService {
  @Autowired private IFileInfoService fileInfoService;
  @Autowired private StocksAuditService stocksAuditService;
  @Autowired private StocksTakeawayDetailService stocksTakeawayDetailService;

  @Value("${yswz.ftp.takeawayInfo}")
  private String path;

  /**
   * 保存外卖出库信息
   *
   * @param takeawayInfoDto 出库信息
   * @param fileIds 文件
   * @return Boolean.TRUE
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean saveTakeawayInfo(
      StocksTakeawayInfoDto takeawayInfoDto, List<MultipartFile> fileIds) {
    if (CollectionUtils.isEmpty(takeawayInfoDto.getTakeawayDetails())) {
      throw new RuntimeException("明细单数据不能为空");
    }
    String baseNo = "WMCK";
    // 创建一个Date对象，获取当前时间
    Date now = new Date();
    String strDateFormat = "yyyyMM";
    String startNo = baseNo + DateFormatUtils.format(now, strDateFormat);
    // 获取当天最大的采样编号
    String sampleNo = baseMapper.getTakeawayMaxNo(startNo);

    int firstNum = 1;
    if (sampleNo.length() == 14) {
      firstNum = Integer.parseInt(sampleNo.substring(11)) + 1;
    }
    // 补全数字，不足用0补位
    String endNo = String.format("%04d", firstNum);

    String outstockNo = startNo + endNo;

    StocksTakeawayInfo data = new StocksTakeawayInfo();
    BeanUtils.copyProperties(takeawayInfoDto, data);
    // 赋值单号
    data.setTakeawayCode(outstockNo);
    // 默认入库人为当前登录人
    data.setTakeawayUser(SecurityUtils.getUserId());
    data.setTakeawayDept(SecurityUtils.getUserDeptId());

    // 附件
    data.setAnnex(
        fileInfoService.uploadFiles(
            fileIds, data.getAnnex(), data.getAnnex(), path, "TF_STOCKS_OUTSTOCK_INFO"));

    data.setTakeawayStatus(StocksAuditEnum.NOT_REVIEWED.getCode());
    this.save(data);
    // 保存入库明细数据
    for (StocksTakeawayDetail takeawayDetail : takeawayInfoDto.getTakeawayDetails()) {
      takeawayDetail.setTakeawayId(data.getId());
      stocksTakeawayDetailService.save(takeawayDetail);
    }
    return true;
  }

  /**
   * 通过id批量删除外卖出库信息
   *
   * @param ids id
   * @return Boolean.TRUE
   */
  @Override
  public boolean removeByIds(String ids) {
    if (StringUtils.isEmpty(ids)) {
      throw new ToflyDeniedException("未选择出库记录");
    }
    List<String> idList = Arrays.asList(ids.split(","));
    // 查询出库记录
    List<StocksTakeawayInfo> infoList =
        list(
            Wrappers.<StocksTakeawayInfo>lambdaQuery()
                .in(StocksTakeawayInfo::getId, idList)
                .select(
                    StocksTakeawayInfo::getId,
                    StocksTakeawayInfo::getTakeawayCode,
                    StocksTakeawayInfo::getTakeawayStatus));
    // 是否存在未审核以外的记录
    String msg =
        infoList.stream()
            .filter(
                info -> !info.getTakeawayStatus().equals(StocksAuditEnum.NOT_REVIEWED.getCode()))
            .map(StocksTakeawayInfo::getTakeawayCode)
            .collect(Collectors.joining(","));
    if (!StringUtils.isEmpty(msg)) {
      throw new ToflyDeniedException(msg + "并未处于未审核接口不可删除");
    }
  //清除审核记录
    stocksAuditService.remove(
        Wrappers.<StocksAudit>lambdaQuery()
            .eq(StocksAudit::getAuditType, StocksAuditEnum.TAKEAWAY_OUT_STOCK_TYPE.getCode())
            .in(StocksAudit::getTableId, idList));

    // 删除记录
    removeByIds(idList);
    // 删除详情表
    stocksTakeawayDetailService.remove(
        Wrappers.<StocksTakeawayDetail>lambdaQuery()
            .in(StocksTakeawayDetail::getTakeawayId, idList));
    return true;
  }

  /**
   * 编辑外卖出库信息
   *
   * @param takeawayInfo 外卖出库信息
   * @param fileIds 文件
   * @return Boolean.TRUE
   */
  @Override
  @Transactional(rollbackFor = Exception.class)
  public boolean editTakeawayStockInfo(
      StocksTakeawayInfoDto takeawayInfo, List<MultipartFile> fileIds) {

    StocksTakeawayInfo takeaway = this.getById(takeawayInfo.getId());
    if (takeaway == null) {
      throw new RuntimeException("修改，参数Id错误");
    }
    if (!takeaway.getCreateUser().equals(SecurityUtils.getUserId())) {
      throw new RuntimeException("只能修改自己创建的数据");
    }
    Long takeawayStatus = takeaway.getTakeawayStatus();
    if (!takeawayStatus.equals(StocksAuditEnum.NOT_REVIEWED.getCode())
        && !takeawayStatus.equals(StocksAuditEnum.HSIANG_HSIEN.getCode())) {
      throw new ToflyDeniedException("该出库单正在审核或审核完成，无法修改");
    }

    takeawayInfo.setTakeawayCode(takeaway.getTakeawayCode());
    StocksTakeawayInfo data = new StocksTakeawayInfo();
    BeanUtils.copyProperties(takeawayInfo, data);

    data.setAnnex(
        fileInfoService.uploadFiles(
            fileIds, data.getAnnex(), takeaway.getAnnex(), path, "TF_STOCKS_TAKEAWAY_INFO"));

    this.updateById(data);

    stocksTakeawayDetailService.remove(
        Wrappers.<StocksTakeawayDetail>lambdaQuery()
            .eq(StocksTakeawayDetail::getTakeawayId, data.getId()));

    for (StocksTakeawayDetail outstockDetail : takeawayInfo.getTakeawayDetails()) {
      outstockDetail.setStockInfoId(data.getId());
      stocksTakeawayDetailService.save(outstockDetail);
    }
    return true;
  }
}
