package com.tofly.yswzMaterials.service.impl;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.yswzMaterials.custom.service.CallApiService;
import com.tofly.yswzMaterials.em.JinDieUrlEnum;
import com.tofly.yswzMaterials.entity.Storage;
import com.tofly.yswzMaterials.entity.StorageDetail;
import com.tofly.yswzMaterials.entity.vo.StorageDetailParamVo;
import com.tofly.yswzMaterials.entity.vo.StorageParamVo;
import com.tofly.yswzMaterials.mapper.StorageMapper;
import com.tofly.yswzMaterials.service.StorageDetailService;
import com.tofly.yswzMaterials.service.StorageService;
import lombok.AllArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author Bubble
 */
@Service
@AllArgsConstructor
public class StorageServiceImpl extends ServiceImpl<StorageMapper, Storage> implements StorageService {

    private final StorageDetailService storageDetailService;
    private final CallApiService callApiService;

    /**
     * 添加外购入库
     *
     * @param storage storage
     * @return boolean
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertStorage(Storage storage) {
        List<StorageDetail> detailList = storage.getDetailList();
        //添加入库单
        save(storage);
        //添加入库单详情
        detailList.forEach(detail->{
            detail.setStorageId(storage.getId());
            storageDetailService.save(detail);
        });
        List<StorageDetailParamVo> detailVoList = new ArrayList<>();
        //格式转换
        for (StorageDetail storageDetail : detailList) {
            StorageDetailParamVo vo = new StorageDetailParamVo();
            BeanUtils.copyProperties(storageDetail,vo);
            detailVoList.add(vo);
        }
        StorageParamVo storageParamVo = new StorageParamVo();
        storage.setDetailList(null);
        BeanUtils.copyProperties(storage,storageParamVo);
        storageParamVo.setDetailList(detailVoList);
        storageParamVo.setFDate(DateUtil.format(storage.getFDate(),"yyyy-MM-dd HH:mm:ss"));
        storageParamVo.setBillerIdName("Administrator");
        //body信息
        String body = JSONObject.toJSONString(storageParamVo);
        //获取token信息
        String tokenInfo = callApiService.getToken();
        //获取url地址
        String url = JinDieUrlEnum.SAVE_UPDATE_STORAGE_URL.getName() + "?token=" + tokenInfo;
        Map apiResult = callApiService.getApiResult(body, url);
        System.out.println(JSONObject.toJSONString(apiResult));
        return true;
    }

    /**
     * 获取外购入库详情
     *
     * @param code code
     * @return StorageParamVo
     */
    @Override
    public StorageParamVo getStorageInfo(String code) {
        String tokenInfo = callApiService.getToken();
        String url = JinDieUrlEnum.GET_STORAGE_URL.getName() + "?token=" + tokenInfo + "&json=" + code;
        Map apiResult = callApiService.getApiResult(null, url);
        if (CollectionUtils.isNotEmpty(apiResult)){
            return JSONObject.parseObject(String.valueOf(apiResult.get("Message")), StorageParamVo.class);
        }
        return null;
    }
}
